import {fetchAuthenticated} from '@/helpers/retrieveData';
import {useLoginStore} from '@/stores/login';
import type {
  GroupListResponseModel,
  ListViewConfig,
  SchoolClassDetailResponseModel,
  WorkGroupDetailResponseModel,
} from '@/helpers/models';
import type {UGridRow} from '@univention/univention-veb';
import {SubElement} from '@univention/univention-veb';


const fetchListViewConfigMocked = (): ListViewConfig => {
  const _config: ListViewConfig = {
    url: 'url',
    allowedGlobalActions: ['add'],
    searchFormOptions: {
      schools: [
        {
          value: 'DEMOSCHOOL',
          label: 'DEMOSCHOOL',
        }, {
          value: 'School1',
          label: 'Schule 1',
        },
      ],
      divisClasstype: [
        {
          value: 'class',
          label: 'Class',
        },
        {
          value: 'offer',
          label: 'Offer',
        },
      ],
      activationStates: [
        {
          value: 'disabled',
          label: 'Disabled',
        },
        {
          value: 'enabled',
          label: 'Enabled',
        },
      ],
      isLmsGroup: [
        {
          value: 'disabled',
          label: 'Disabled',
        },
        {
          value: 'enabled',
          label: 'Enabled',
        },
      ],
      isIservGroup: [
        {
          value: 'disabled',
          label: 'Disabled',
        },
        {
          value: 'enabled',
          label: 'Enabled',
        },
      ],
    },
    columns: [
      {
        label: 'School year',
        attribute: 'idiSchoolyear',
      },
      {
        label: 'School name',
        attribute: 'displayName',
      },
      {
        label: 'Name',
        attribute: 'name',
      },
      {
        label: 'Group type',
        attribute: 'divisClasstype',
      },
      {
        label: 'Source UID',
        attribute: 'groupSourceUid',
      },
      {
        label: 'Creation date',
        attribute: 'createTimestamp',
        formatter: 'date',
      },
      {
        label: 'Date of last change',
        attribute: 'modifyTimestamp',
        formatter: 'date',
      },
      {
        label: 'Record UID',
        attribute: 'recordUid',
      },
      {
        label: 'Activation date',
        attribute: 'activationDate',
        formatter: 'date',
      },
      {
        label: 'Expiry date',
        attribute: 'expiryDate',
        formatter: 'date',
      },
      {
        label: 'Scheduled deletion date',
        attribute: 'scheduledDeletionDate',
        formatter: 'date',
      },
      {
        label: 'LMS enabled',
        attribute: 'isLmsGroup',
        formatter: 'boolean:LMS enabled:LMS disabled',
      },
      {
        label: 'IServ enabled',
        attribute: 'isIservGroup',
        formatter: 'boolean:IServ enabled:IServ disabled',
      },
    ],
  };
  console.debug('mock config', _config);
  return _config;
};

export const fetchListViewConfig = async(): Promise<ListViewConfig> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchListViewConfig. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchListViewConfigMocked());
      }, 1000);
    });
  }

  const loginStore = useLoginStore();
  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/pages/listView/settings/${loginStore.username}`);
  return await answer.json();
};


const fetchGroupsMocked = (): GroupListResponseModel[] => {
  /* eslint-disable camelcase */
  const group: GroupListResponseModel = {
    id: 'school1-group1',
    allowedActions: ['edit', 'delete'],
    schoolGroupType: 'workgroup',
    attributes: {
      idiSchoolyear: {
        value: 'Year 3',
        access: 'read',
      },
      displayName: {
        value: 'Group 1',
        access: 'read',
      },
      name: {
        value: 'school1-group1',
        access: 'read',
      },
      divisClasstype: {
        value: 'Group type A',
        access: 'read',
      },
      groupSourceUid: {
        value: 'sourceID',
        access: 'read',
      },
      createTimestamp: {
        value: '2020-10-5',
        access: 'read',
      },
      modifyTimestamp: {
        value: '2020-10-05',
        access: 'read',
      },
      recordUid: {
        value: 'recordUID',
        access: 'read',
      },
      activationDate: {
        value: '2020-10-05',
        access: 'read',
      },
      expiryDate: {
        value: '2020-10-05',
        access: 'none',
      },
      scheduledDeletionDate: {
        value: '2020-10-05',
        access: 'none',
      },
      isLmsGroup: {
        value: true,
        access: 'none',
      },
      isIservGroup: {
        value: false,
        access: 'none',
      },
    },
  };
  /* eslint-enable camelcase */

  const _groups: GroupListResponseModel[] = [group];

  for (let x = 2; x < 500; x++) {
    const y = JSON.parse(JSON.stringify(group));
    y.id = `school1-group${x}`;
    y.attributes.name.value = y.id;
    if (x % 2 === 0) {
      y.allowedActions = ['edit'];
    }
    y.schoolGroupType = 'school_class';
    _groups.push(y);
  }
  console.log('mock groups: ', _groups);
  return _groups;
};

export const fetchGroups = async(
  school: string,
  schoolYear: string,
  divisClassType: string,
  source: string,
  activationState: string,
  isLmsGroup: string,
  iServEnabled: string,
  quickSearch: string,
): Promise<GroupListResponseModel[]> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchGroups. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchGroupsMocked());
      }, 1000);
    });
  }

  const urlSearchParams = new URLSearchParams({});
  if (schoolYear !== '') {
    urlSearchParams.append('schoolYear', schoolYear);
  }
  if (divisClassType !== '') {
    urlSearchParams.append('divisClassType', divisClassType);
  }
  if (source !== '') {
    urlSearchParams.append('source', source);
  }
  if (activationState !== '') {
    urlSearchParams.append('activationState', activationState);
  }
  if (isLmsGroup !== '') {
    urlSearchParams.append('isLmsGroup', (isLmsGroup === 'enabled').toString());
  }
  if (iServEnabled !== '') {
    urlSearchParams.append('isIservGroup', (iServEnabled === 'enabled').toString());
  }
  if (quickSearch !== '') {
    urlSearchParams.append('quickSearch', quickSearch);
  }
  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/groups/search/${school}?${urlSearchParams.toString()}`);

  return await answer.json();
};


export const deleteGroups = async(groups: GroupListResponseModel[]): Promise<string[]> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('deleteGroups. VITE_USE_REAL_BACKEND===false in DEV mode. Doing nothing');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve([groups[0]?.id as string]);
      }, 1000);
    });
  }

  const fails: string[] = [];
  const promises: Promise<Response>[] = [];
  for (const group of groups) {
    const promise = fetchAuthenticated(`/ucsschool/bff-groups/v1/groups/${group.schoolGroupType}/${group.id}`, {
      method: 'DELETE',
    });
    promise.then(res => {
      if (!res.ok) {
        fails.push(group.id);
      }
    });
    promises.push(promise);
  }
  await Promise.all(promises);
  return fails;
};

const fetchMembersMocked = (): UGridRow[] => [
  {
    id: 'max.mustermann',
    attributes: {
      username: {
        value: 'max.mustermann',
        access: 'read',
      },
      firstname: {
        value: 'Max',
        access: 'read',
      },
      lastname: {
        value: 'Mustermann',
        access: 'read',
      },
      role: {
        value: 'student',
        access: 'read',
      },
    },
  },
  {
    id: 'hans.peter',
    attributes: {
      username: {
        value: 'hans.peter',
        access: 'read',
      },
      firstname: {
        value: 'Hans',
        access: 'read',
      },
      lastname: {
        value: 'Peter',
        access: 'read',
      },
      role: {
        value: 'student',
        access: 'read',
      },
    },
  },
  {
    id: 'walter.scholl',
    attributes: {
      username: {
        value: 'walter.scholl',
        access: 'read',
      },
      firstname: {
        value: 'Walter',
        access: 'read',
      },
      lastname: {
        value: 'Scholl',
        access: 'read',
      },
      role: {
        value: 'student',
        access: 'read',
      },
    },
  },
];

export const fetchMembers = async(school: string, values: Record<string, unknown>): Promise<UGridRow[] | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchMembers. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchMembersMocked());
      }, 1000);
    });
  }

  const urlSearchParams = new URLSearchParams({});
  for (const key in values) {
    const value = values[key];
    if (typeof value === 'string' && value !== '') {
      urlSearchParams.append(key, value);
    }
  }
  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/users/${school}?${urlSearchParams.toString()}`);
  if (!answer.ok) {
    // FIXME differentiate between 404 (school class does not exist) and other (backend failed in some way)
    // ... and other error cases (e.g. login token expired?)
    return null;
  }
  return await answer.json();
};


export const getDetailResponseModelMocked = (): WorkGroupDetailResponseModel | SchoolClassDetailResponseModel => ({
  url: '',
  allowedActions: ['save'],
  pages: [
    {
      label: 'General',
      name: 'general',
      fieldsets: [
        {
          label: 'Basic settings',
          name: 'general__basic',
          rows: [
            [
              {
                type: 'UInputText',
                props: {
                  name: 'idiSchoolyear',
                  label: 'School year',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputText',
                props: {
                  name: 'description',
                  label: 'Description',
                  required: true,
                  access: 'write',
                },
              },
            ],
            [
              {
                type: 'UInputText',
                props: {
                  name: 'name',
                  label: 'Name',
                  required: false,
                  access: 'write',
                },
              },
            ],
            [
              {
                type: 'UMultiObjectSelect',
                props: {
                  name: 'users',
                  label: 'Users',
                  access: 'write',
                  searchStandby: false,
                  initialRows: [
                    {
                      id: 'max.mustermann',
                      attributes: {
                        username: {
                          value: 'max.mustermann',
                          access: 'read',
                        },
                        firstname: {
                          value: 'Max',
                          access: 'read',
                        },
                        lastname: {
                          value: 'Mustermann',
                          access: 'read',
                        },
                        role: {
                          value: 'student',
                          access: 'read',
                        },
                      },
                    },
                  ],
                  columns: [
                    {
                      label: 'Username',
                      attribute: 'username',
                    },
                    {
                      label: 'Firstname',
                      attribute: 'firstname',
                    },
                    {
                      label: 'Lastname',
                      attribute: 'lastname',
                    },
                    {
                      label: 'Role',
                      attribute: 'role',
                    },
                  ],
                  searchRows: [],
                  searchElements: [
                    {
                      type: SubElement.UComboBox,
                      props: {
                        label: 'Role',
                        name: 'role',
                        options: [
                          {
                            value: 'role1',
                            label: 'Role 1',
                          },
                          {
                            value: 'role2',
                            label: 'Role 2',
                          },
                        ],
                      },
                    },
                    {
                      type: SubElement.UComboBox,
                      props: {
                        label: 'Schoolclass',
                        name: 'schoolClass',
                        options: [
                          {
                            value: 'class1',
                            label: 'Class 1',
                          },
                          {
                            value: 'class2',
                            label: 'Class 2',
                          },
                        ],
                      },
                    },
                    {
                      type: SubElement.UComboBox,
                      props: {
                        label: 'Workgroup',
                        name: 'workGroup',
                        options: [
                          {
                            value: 'workgroup1',
                            label: 'Workgroup 1',
                          },
                          {
                            value: 'workgroup2',
                            label: 'Workgroup 2',
                          },
                        ],
                      },
                    },
                    {
                      type: SubElement.UInputText,
                      props: {
                        label: 'Suche',
                        name: 'quickSearch',
                      },
                    },
                  ],
                  searchColumns: [
                    {
                      label: 'Username',
                      attribute: 'username',
                    },
                    {
                      label: 'Firstname',
                      attribute: 'firstname',
                    },
                    {
                      label: 'Lastname',
                      attribute: 'lastname',
                    },
                    {
                      label: 'Role',
                      attribute: 'role',
                    },
                  ],
                },
              },
            ],
            [
              {
                type: 'UComboBox',
                props: {
                  name: 'divisClasstype',
                  label: 'Group type',
                  required: false,
                  access: 'write',
                  options: [
                    {
                      label: 'Class',
                      value: 'class',
                    },
                    {
                      label: 'Offer',
                      value: 'offer',
                    },
                  ],
                },
              },
              {
                type: 'UComboBox',
                props: {
                  name: 'divisCoursetype',
                  label: 'Course type',
                  required: false,
                  access: 'write',
                  options: [
                    {
                      label: 'Course 1',
                      value: 'type1',
                    },
                    {
                      label: 'Course 2',
                      value: 'type2',
                    },
                  ],
                },
              },
            ],
            [
              {
                type: 'UInputText',
                props: {
                  label: 'Source',
                  name: 'groupSourceId',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputText',
                props: {
                  label: 'Record ID',
                  name: 'recordUID',
                  required: false,
                  access: 'write',
                },
              },
            ],
            [
              {
                type: 'UInputCheckbox',
                props: {
                  label: 'LMS enabled',
                  name: 'isLmsGroup',
                  required: false,
                  access: 'write',
                },
              },
            ],
            [
              {
                type: 'UInputCheckbox',
                props: {
                  label: 'IServ enabled',
                  name: 'isIservGroup',
                  required: false,
                  access: 'write',
                },
              },
            ],
          ],
        },
      ],
    },
    {
      label: 'Account',
      name: 'account',
      fieldsets: [
        {
          label: 'General',
          name: 'account__general',
          rows: [
            [
              {
                type: 'UInputDate',
                props: {
                  label: 'Creation date',
                  name: 'createTimestamp',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputDate',
                props: {
                  label: 'Date of last change',
                  name: 'modifyTimestamp',
                  required: false,
                  access: 'write',
                },
              },
            ],
            [
              {
                type: 'UInputDate',
                props: {
                  label: 'Activation date',
                  name: 'divisStartDate',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputDate',
                props: {
                  label: 'Expiration date',
                  name: 'divisEndDate',
                  required: false,
                  access: 'write',
                },
              },
            ],
            [
              {
                type: 'UInputDate',
                props: {
                  label: 'Scheduled deletion date',
                  name: 'purgeDate',
                  required: false,
                  access: 'write',
                },
              },
            ],
          ],
        },
        {
          label: 'SAML settings',
          name: 'account__samlSettings',
          rows: [
            {
              type: 'UMultiSelect',
              props: {
                name: 'serviceprovidergroup',
                label: 'SAML service provider',
                required: false,
                columns: [
                  {
                    label: 'Select all',
                    attribute: 'label',
                  },
                ],
                rows: [
                  {
                    id: 'sp1',
                    attributes: {
                      label: {
                        value: 'Service provider 1',
                        access: 'read',
                      },
                    },
                  },
                  {
                    id: 'sp2',
                    attributes: {
                      label: {
                        value: 'Service provider 2',
                        access: 'read',
                      },
                    },
                  },
                  {
                    id: 'sp3',
                    attributes: {
                      label: {
                        value: 'Service provider 3',
                        access: 'read',
                      },
                    },
                  },
                ],
              },
            },
          ],
        },
      ],
    },
  ],
  values: {
    idiSchoolyear: '2020',
    description: 'description',
    name: 'username',
    users: ['max.mustermann'],
    divisClasstype: 'class',
    divisCoursetype: 'type1',
    groupSourceId: 'source',
    recordUID: 'record id',
    isLmsGroup: true,
    isIservGroup: true,
    createTimestamp: '2020-01-01',
    modifyTimestamp: '2020-01-01',
    divisStartDate: '2020-01-01',
    divisEndDate: '2020-01-01',
    purgeDate:  '2020-01-01',
    serviceprovidergroup: [],
  },
});
