import {useLoginStore} from '@/stores/login';
import type {
  AddViewConfig,
  FormValues,
  WorkGroupDetailResponseModel,
} from '../models';
import {fetchAuthenticated} from '../retrieveData';
import {getDetailResponseModelMocked} from '@/helpers/dataAccess';


const fetchWorkGroupAddViewConfigMocked = (): AddViewConfig => {
  const config: AddViewConfig = {
    pages: [
      {
        label: 'General',
        name: 'general',
        fieldsets: [
          {
            label: 'Basic settings',
            name: 'general__basicSettings',
            rows: [
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'displayName',
                    label: 'Display name',
                    required: true,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputText',
                  props: {
                    name: 'name',
                    label: 'Name',
                    required: true,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UComboBox',
                  props: {
                    name: 'school',
                    label: 'School',
                    required: true,
                    access: 'write',
                    options: [
                      {
                        label: 'Schule 1',
                        value: 'school1',
                      },
                      {
                        label: 'Schule 2',
                        value: 'school2',
                      },
                    ],
                  },
                },
              ],
              [
                {
                  type: 'UInputDate',
                  props: {
                    name: 'divisStartDate',
                    label: 'Creation date',
                    required: false,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputDate',
                  props: {
                    name: 'divisEndDate',
                    label: 'Expiry date',
                    required: false,
                    access: 'write',
                  },
                },
              ],
            ],
          },
        ],
      },
    ],
  };
  console.debug('mock config', config);
  return config;
};
export const fetchWorkGroupAddViewConfig = async(): Promise<AddViewConfig | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchWorkGroupAddViewConfig. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchWorkGroupAddViewConfigMocked());
      }, 1000);
    });
  }
  const loginStore = useLoginStore();
  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/pages/addView/settings/${loginStore.username}`);

  return await answer.json();
};

export const createWorkGroup = async(values: FormValues): Promise<boolean> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('createWorkGroup. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(true);
      }, 1000);
    });
  }

  const answer = await fetchAuthenticated('/ucsschool/bff-groups/v1/groups/workgroup', {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
    },
    body: JSON.stringify(values),
  });

  return answer.ok;
};

const fetchWorkGroupMocked = (): WorkGroupDetailResponseModel => {
  const workGroup: WorkGroupDetailResponseModel = getDetailResponseModelMocked();
  console.debug('mock workgroup', workGroup);
  return workGroup;
};

export const fetchWorkGroup = async(workGroup: string): Promise<WorkGroupDetailResponseModel | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchWorkGroup. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchWorkGroupMocked());
      }, 1000);
    });
  }

  // TODO adjust url again when issue https://git.knut.univention.de/univention/ucsschool-components/ui-groups/-/issues/47 is done
  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/groups/workgroup/detail/${workGroup}`);
  if (!answer.ok) {
    // FIXME differentiate between 404 (school class does not exist) and other (backend failed in some way)
    // ... and other error cases (e.g. login token expired?)
    return null;
  }
  return await answer.json();
};

export const updateWorkGroup = async(url: string, values: FormValues): Promise<string> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('updateWorkGroup. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    // TODO have the fail case in some way too
    return new Promise(resolve => {
      setTimeout(() => {
        resolve('');
      }, 1000);
    });
  }

  const fixedProtocolUrl = url.replace(/https?:/, location.protocol).replace('detail/', '');
  const answer = await fetchAuthenticated(`${fixedProtocolUrl}`, {
    method: 'PATCH',
    headers: {
      'Content-Type': 'application/json',
    },
    body: JSON.stringify(values),
  });

  if (!answer.ok) {
    if (answer.status === 404) {
      return 'dataAccess.updateWorkGroup.error.404';
    }
    if (answer.status >= 500 && answer.status < 600) {
      return 'dataAccess.updateWorkGroup.error.5xx';
    }
    if (answer.status >= 400 && answer.status < 500) {
      return 'dataAccess.updateWorkGroup.error.4xx';
    }
  }
  return '';
};
